//
// Created by robin on 7/22/16.
//

#include <libnfc_cpptools/FreeFareTagClassic.h>
#include "libnfc_cpptools/FreeFareDevice.h"
#include "FreeFareDeviceInternal.h"
#include "Tags/Classic/FreeFareTagClassicInternal.h"

namespace LibNfc
{
namespace FreeFare
{

FreeFareDevice::FreeFareDevice(std::shared_ptr<LibNfc::Core::NfcDevice> device)
    : _device(device),
      _freeFareDevice(std::make_shared<FreeFareDeviceInternal>(_device->getDevice()))
{
}

LibNfc::Utils::Result<std::vector<std::shared_ptr<FreeFareTag>>> FreeFareDevice::getTags() const
{
    if (!_device->isOpened()) {
        return LibNfc::Utils::Result<std::vector<std::shared_ptr<FreeFareTag>>>::error("Device is not opened");
    }
    auto tagsResult = _freeFareDevice->getTags();
    if (!tagsResult) {
        return LibNfc::Utils::Result<std::vector<std::shared_ptr<FreeFareTag>>>::error(tagsResult);
    }
    auto tags = tagsResult.getData();
    std::vector<std::shared_ptr<FreeFareTag>> tagsBusiness;
    for (auto tag : tags) {
        if (tag->getType() == MIFARE_CLASSIC_1K || tag->getType() == MIFARE_CLASSIC_4K) {
            tagsBusiness.push_back(std::make_shared<FreeFareTagClassic>(std::dynamic_pointer_cast<FreeFareTagClassicInternal>(tag)));
        }
        else {
            tagsBusiness.push_back(std::make_shared<FreeFareTag>(tag));
        }
    }
    return LibNfc::Utils::Result<std::vector<std::shared_ptr<FreeFareTag>>>::ok(tagsBusiness);
}

}; // FreeFare
}; // LibNfc
