//
// Created by robin on 6/19/16.
//

#include <Tags/Classic/FreeFareTagClassicInternal.h>
#include "FreeFareDeviceInternal.h"

namespace LibNfc
{
namespace FreeFare
{

FreeFareDeviceInternal::FreeFareDeviceInternal(std::shared_ptr<LibNfc::Core::NfcDeviceInternal> device)
    : _device(device)
{
}

LibNfc::Utils::Result<std::vector<std::shared_ptr<FreeFareTagInternal>>> FreeFareDeviceInternal::getTags()
{
    FreefareTag* tags = freefare_get_tags(_device->getDevice());
    if (!tags) {
        return LibNfc::Utils::Result<std::vector<std::shared_ptr<FreeFareTagInternal>>>::error("Failed to get MIFARE tags");
    }
    std::vector<std::shared_ptr<FreeFareTagInternal>> tagList;
    for (size_t i = 0; tags[i] != 0; ++i) {
        auto type = freefare_get_tag_type(tags[i]);
        if (type == MIFARE_CLASSIC_1K || type == MIFARE_CLASSIC_4K) {
            tagList.push_back(std::make_shared<FreeFareTagClassicInternal>(tags[i]));
        }
        else {
            tagList.push_back(std::make_shared<FreeFareTagInternal>(tags[i]));
        }
    }
    return LibNfc::Utils::Result<std::vector<std::shared_ptr<FreeFareTagInternal>>>::ok(tagList);
}

}; // FreeFare
}; // LibNfc
