//
// Created by robin on 7/21/16.
//

#include <cstring>
#include <iostream>
#include "libnfc_cpptools/FreeFareClassicAccessBits.h"
#include "libnfc_cpptools/StringUtils.h"
#include "libnfc_cpptools/ArrayUtils.h"

namespace LibNfc
{
namespace FreeFare
{

const char FreeFareClassicAccessBits::nonInvertedBitPosition[4][4] = {
        {0,  0,  0,  0 },
        {11, 10, 9,  8 },
        {23, 22, 21, 20},
        {19, 18, 17, 16}};

const char FreeFareClassicAccessBits::invertedBitPosition[4][4] = {
        {0,  0,  0,  0 },
        {7,  6,  5,  4 },
        {3,  2,  1,  0},
        {15, 14, 13, 12}};

FreeFareClassicAccessBits::FreeFareClassicAccessBits()
    : _bits("\xff\x0f\00\x00", 4)
{
}

FreeFareClassicAccessBits::FreeFareClassicAccessBits(const std::string &bits)
    : _bits(LibNfc::Utils::StringUtils::ensureSize(bits, 4))
{
}

char FreeFareClassicAccessBits::getUserData() const
{
    return _bits[_bits.length() - 1];
}

void FreeFareClassicAccessBits::setUserData(const char &data)
{
    _bits[_bits.length() - 1] = data;
}

void FreeFareClassicAccessBits::setBit(int i, int j, const bool& value)
{
    char buf[_bits.length()];
    memcpy(buf, _bits.c_str(), _bits.length());

    LibNfc::Utils::ArrayUtils::setArrayBit(buf, nonInvertedBitPosition[i][j], value);
    LibNfc::Utils::ArrayUtils::setArrayBit(buf, invertedBitPosition[i][j], !value);

    _bits = std::string(buf, _bits.length());
}

bool FreeFareClassicAccessBits::getBit(int i, int j) const
{
    const char* buf = _bits.c_str();
    return LibNfc::Utils::ArrayUtils::getArrayBit(buf, nonInvertedBitPosition[i][j]) && !LibNfc::Utils::ArrayUtils::getArrayBit(buf, invertedBitPosition[i][j]);
}

std::string FreeFareClassicAccessBits::getBits() const
{
    return _bits;
}

bool FreeFareClassicAccessBits::canKeyAReadBlock(int block) const
{
    bool c1 = getBit(1, block);
    bool c2 = getBit(2, block);
    bool c3 = getBit(3, block);
    return !c3 || (!c1 && !c2 && c3);
}

bool FreeFareClassicAccessBits::canKeyBReadBlock(int block) const
{
    bool c1 = getBit(1, block);
    bool c2 = getBit(2, block);
    bool c3 = getBit(3, block);
    return !c1 || !c2 || !c3;
}

bool FreeFareClassicAccessBits::canKeyAWriteBlock(int block) const
{
    bool c1 = getBit(1, block);
    bool c2 = getBit(2, block);
    bool c3 = getBit(3, block);
    return !c1 && !c2 && !c3;
}

bool FreeFareClassicAccessBits::canKeyBWriteBlock(int block) const
{
    bool c1 = getBit(1, block);
    bool c2 = getBit(2, block);
    bool c3 = getBit(3, block);
    return (!c2 && !c3) || (c1 && !c3) || (!c1 && c2 && c3);
}

bool FreeFareClassicAccessBits::canKeyAIncrementBlock(int block) const
{
    bool c1 = getBit(1, block);
    bool c2 = getBit(2, block);
    bool c3 = getBit(3, block);
    return !c1 && !c2 && !c3;
}

bool FreeFareClassicAccessBits::canKeyBIncrementBlock(int block) const
{
    bool c1 = getBit(1, block);
    bool c2 = getBit(2, block);
    bool c3 = getBit(3, block);
    return (!c1 && !c2 && !c3) || (c1 && c2 && !c3);
}

bool FreeFareClassicAccessBits::canKeyADecrementBlock(int block) const
{
    bool c1 = getBit(1, block);
    bool c2 = getBit(2, block);
    bool c3 = getBit(3, block);
    return (!c1 && !c2) || (c1 && c2 && !c3);
}

bool FreeFareClassicAccessBits::canKeyBDecrementBlock(int block) const
{
    bool c1 = getBit(1, block);
    bool c2 = getBit(2, block);
    bool c3 = getBit(3, block);
    return (!c1 && !c2) || (c1 && c2 && !c3);
}

bool FreeFareClassicAccessBits::canKeyAReadKeyATrailer() const
{
    return false;
}

bool FreeFareClassicAccessBits::canKeyBReadKeyATrailer() const
{
    return false;
}

bool FreeFareClassicAccessBits::canKeyAWriteKeyATrailer() const
{
    bool c1 = getBit(1, 3);
    bool c2 = getBit(2, 3);
    return !c1 && !c2;
}

bool FreeFareClassicAccessBits::canKeyBWriteKeyATrailer() const
{
    bool c1 = getBit(1, 3);
    bool c2 = getBit(2, 3);
    bool c3 = getBit(3, 3);
    return (!c1 && c2 && c3) || (c1 && !c2 && !c3);
}

bool FreeFareClassicAccessBits::canKeyAReadAccessBitsTrailer() const
{
    return true;
}

bool FreeFareClassicAccessBits::canKeyBReadAccessBitsTrailer() const
{
    bool c1 = getBit(1, 3);
    bool c2 = getBit(2, 3);
    bool c3 = getBit(3, 3);
    return c1 || (c2 && c3);
}

bool FreeFareClassicAccessBits::canKeyAWriteAccessBitsTrailer() const
{
    bool c1 = getBit(1, 3);
    bool c2 = getBit(2, 3);
    bool c3 = getBit(3, 3);
    return !c1 && !c2 && c3;
}

bool FreeFareClassicAccessBits::canKeyBWriteAccessBitsTrailer() const
{
    bool c1 = getBit(1, 3);
    bool c2 = getBit(2, 3);
    bool c3 = getBit(3, 3);
    return (!c1 && c2 && c3) || (c1 && !c2 && c3);
}

bool FreeFareClassicAccessBits::canKeyAReadKeyBTrailer() const
{
    bool c1 = getBit(1, 3);
    bool c2 = getBit(2, 3);
    bool c3 = getBit(3, 3);
    return (!c1 && !c2) || (!c1 && !c3);
}

bool FreeFareClassicAccessBits::canKeyBReadKeyBTrailer() const
{
    return false;
}

bool FreeFareClassicAccessBits::canKeyAWriteKeyBTrailer() const
{
    bool c1 = getBit(1, 3);
    bool c2 = getBit(2, 3);
    return !c1 && !c2;
}

bool FreeFareClassicAccessBits::canKeyBWriteKeyBTrailer() const
{
    bool c1 = getBit(1, 3);
    bool c2 = getBit(2, 3);
    bool c3 = getBit(3, 3);
    return (!c1 && c2 && c3) || (c1 && !c2 && !c3);
}

}; // FreeFare
}; // LibNfc
