//
// Created by robin on 7/22/16.
//

#include "libnfc_cpptools/StringUtils.h"
#include "libnfc_cpptools/FreeFareClassicSector.h"

namespace LibNfc
{
namespace FreeFare
{

FreeFareClassicSector::FreeFareClassicSector(const std::string &data)
{
    std::string d = LibNfc::Utils::StringUtils::ensureSize(data, 64);
    for (unsigned int i = 0; i < 4; ++i) {
        _blocks[i] = d.substr(16 * i, 16);
        _haveBlocks[i] = true;
    }
}

const std::string &FreeFareClassicSector::getBlock(int block) const
{
    return _blocks[block];
}

std::string FreeFareClassicSector::getKeyA() const
{
    return _blocks[3].substr(0, 6);
}

std::string FreeFareClassicSector::getKeyB() const
{
    return _blocks[3].substr(10, 6);
}

std::string FreeFareClassicSector::getAccessBits() const
{
    return _blocks[3].substr(6, 4);
}

FreeFareClassicAccessBits FreeFareClassicSector::getAccessBitsDbo() const
{
    return FreeFareClassicAccessBits(getAccessBits());
}

void FreeFareClassicSector::setBlock(int block, const std::string &data)
{
    std::string d = LibNfc::Utils::StringUtils::ensureSize(data, 16);
    if (block == 3) {
        setKeyA(d.substr(0, 6));
        setKeyB(d.substr(10, 6));
        setAccessBits(d.substr(6, 4));
    }
    else {
        _haveBlocks[block] = !data.empty();
        _blocks[block] = d;
    }
}

void FreeFareClassicSector::setKeyA(const std::string &key)
{
    std::string k = LibNfc::Utils::StringUtils::ensureSize(key, 6);
    for (int i = 0; i < k.size(); ++i) {
        _blocks[3][i] = k[i];
    }
    _hasKeyA = !key.empty();
}

void FreeFareClassicSector::setKeyB(const std::string &key)
{
    std::string k = LibNfc::Utils::StringUtils::ensureSize(key, 6);
    for (int i = 0; i < k.size(); ++i) {
        _blocks[3][10 + i] = k[i];
    }
    _hasKeyB = !key.empty();
}

void FreeFareClassicSector::setAccessBits(const std::string &accessBits)
{
    std::string a = LibNfc::Utils::StringUtils::ensureSize(accessBits, 4);
    for (int i = 0; i < a.size(); ++i) {
        _blocks[3][6 + i] = a[i];
    }
    _hasAccessBits = !accessBits.empty();
}

void FreeFareClassicSector::setAccessBits(const FreeFareClassicAccessBits &accessBits)
{
    setAccessBits(accessBits.getBits());
}

bool FreeFareClassicSector::hasBlock(int block) const
{
    if (block == 3) {
        return _hasKeyA && _hasKeyB && _hasAccessBits;
    }
    return _haveBlocks[block];
}

bool FreeFareClassicSector::hasKeyA() const
{
    return _hasKeyA;
}

bool FreeFareClassicSector::hasKeyB() const
{
    return _hasKeyB;
}

bool FreeFareClassicSector::hasAccessBits() const
{
    return _hasAccessBits;
}

}; // FreeFare
}; // LibNfc
