//
// Created by robin on 10/8/17.
//

#include "FreeFareTagClassicInternal.h"

namespace LibNfc
{
namespace FreeFare
{

FreeFareTagClassicInternal::FreeFareTagClassicInternal(FreefareTag tag)
        : FreeFareTagInternal(tag)
{
}

LibNfc::Utils::ResultBool FreeFareTagClassicInternal::authenticate(int sector, std::string key, int keyType)
{

    if (mifare_classic_connect(_tag) != 0) {
        return LibNfc::Utils::ResultBool::error("Failed to connect to MIFARE tag");
    }
    int block = mifare_classic_sector_last_block((MifareClassicBlockNumber)sector);
    if (mifare_classic_authenticate(_tag, (MifareClassicBlockNumber)block, (const unsigned char*)key.c_str(),
                                    (MifareClassicKeyType)keyType) != 0) {
        return LibNfc::Utils::ResultBool::error("Failed to authenticate to MIFARE tag");
    }
    mifare_classic_disconnect(_tag);
    return LibNfc::Utils::ResultBool::ok(true);
}

LibNfc::Utils::ResultString FreeFareTagClassicInternal::readBlock(int sector, int block, std::string key, int keyType)
{
    if (mifare_classic_connect(_tag) != 0) {
        return LibNfc::Utils::ResultString::error("Failed to connect to MIFARE tag");
    }
    block = mifare_classic_sector_first_block((MifareClassicBlockNumber)sector) + block;
    if (mifare_classic_authenticate(_tag, (MifareClassicBlockNumber)block, (const unsigned char*)key.c_str(),
                                    (MifareClassicKeyType)keyType) != 0) {
        return LibNfc::Utils::ResultString::error("Failed to authenticate to MIFARE tag");
    }

    MifareClassicBlock data;
    if (mifare_classic_read(_tag, (MifareClassicBlockNumber)block, &data)) {
        return LibNfc::Utils::ResultString::error("Failed to read MIFARE tag data");
    }
    mifare_classic_disconnect(_tag);
    return LibNfc::Utils::ResultString::ok(std::string((const char*)data, sizeof(data)));
}

LibNfc::Utils::ResultBool FreeFareTagClassicInternal::writeBlock(int sector, int block, std::string key, int keyType, const std::string &data)
{
    if (mifare_classic_connect(_tag) != 0) {
        return LibNfc::Utils::ResultBool::error("Failed to connect to MIFARE tag");
    }
    block = mifare_classic_sector_first_block((MifareClassicBlockNumber)sector) + block;
    if (mifare_classic_authenticate(_tag, (MifareClassicBlockNumber)block, (const unsigned char*)key.c_str(),
                                    (MifareClassicKeyType)keyType) != 0) {
        return LibNfc::Utils::ResultBool::error("Failed to authenticate to MIFARE tag");
    }

    if (mifare_classic_write(_tag, (MifareClassicBlockNumber)block, (const unsigned char*)data.c_str())) {
        return LibNfc::Utils::ResultBool::error("Failed to write data to MIFARE tag");
    }
    mifare_classic_disconnect(_tag);
    return LibNfc::Utils::ResultBool::ok(true);
}

int FreeFareTagClassicInternal::getFirstBlock(int sector) const
{
    return mifare_classic_sector_first_block(sector);
}

int FreeFareTagClassicInternal::getLastBlock(int sector) const
{
    return mifare_classic_sector_last_block(sector);
}

int FreeFareTagClassicInternal::getSectorBlockCount(int sector)
{
    return mifare_classic_sector_block_count(sector);
}
}; // FreeFare
}; // LibNfc
